{% extends "lava_scheduler_app/_content.html" %}

{% load django_tables2 %}

{% block content %}
<h2>{{device_type}} Status</h2>

{{running_jobs_num}} jobs running / {{queued_jobs_num}} jobs queued

<h3>Health Job Summary</h3>
{% render_table health_job_summary_table %}

<h3>Devices Overview</h3>
{% render_table devices_table_no_dt %}

{% endblock %}
